# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: model_registry.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import service as _service
from google.protobuf import service_reflection
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from .scalapb import scalapb_pb2 as scalapb_dot_scalapb__pb2
from . import databricks_pb2 as databricks__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='model_registry.proto',
  package='mlflow',
  syntax='proto2',
  serialized_options=_b('\n\024org.mlflow.api.proto\220\001\001\240\001\001\342?\002\020\001'),
  serialized_pb=_b('\n\x14model_registry.proto\x12\x06mlflow\x1a\x15scalapb/scalapb.proto\x1a\x10\x64\x61tabricks.proto\"\x1f\n\x0fRegisteredModel\x12\x0c\n\x04name\x18\x01 \x01(\t\"\xe5\x01\n\x17RegisteredModelDetailed\x12\x31\n\x10registered_model\x18\x01 \x01(\x0b\x32\x17.mlflow.RegisteredModel\x12\x1a\n\x12\x63reation_timestamp\x18\x02 \x01(\x03\x12\x1e\n\x16last_updated_timestamp\x18\x03 \x01(\x03\x12\x0f\n\x07user_id\x18\x04 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\x12\x35\n\x0flatest_versions\x18\x06 \x03(\x0b\x32\x1c.mlflow.ModelVersionDetailed\"R\n\x0cModelVersion\x12\x31\n\x10registered_model\x18\x01 \x01(\x0b\x32\x17.mlflow.RegisteredModel\x12\x0f\n\x07version\x18\x02 \x01(\x03\"\xa0\x02\n\x14ModelVersionDetailed\x12+\n\rmodel_version\x18\x01 \x01(\x0b\x32\x14.mlflow.ModelVersion\x12\x1a\n\x12\x63reation_timestamp\x18\x02 \x01(\x03\x12\x1e\n\x16last_updated_timestamp\x18\x03 \x01(\x03\x12\x0f\n\x07user_id\x18\x04 \x01(\t\x12\x15\n\rcurrent_stage\x18\x05 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x06 \x01(\t\x12\x0e\n\x06source\x18\x07 \x01(\t\x12\x0e\n\x06run_id\x18\x08 \x01(\t\x12*\n\x06status\x18\t \x01(\x0e\x32\x1a.mlflow.ModelVersionStatus\x12\x16\n\x0estatus_message\x18\n \x01(\t\"\x97\x01\n\x15\x43reateRegisteredModel\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a=\n\x08Response\x12\x31\n\x10registered_model\x18\x01 \x01(\x0b\x32\x17.mlflow.RegisteredModel:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xdf\x01\n\x15UpdateRegisteredModel\x12\x37\n\x10registered_model\x18\x01 \x01(\x0b\x32\x17.mlflow.RegisteredModelB\x04\xf8\x86\x19\x01\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x1a=\n\x08Response\x12\x31\n\x10registered_model\x18\x01 \x01(\x0b\x32\x17.mlflow.RegisteredModel:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x89\x01\n\x15\x44\x65leteRegisteredModel\x12\x37\n\x10registered_model\x18\x01 \x01(\x0b\x32\x17.mlflow.RegisteredModelB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xd1\x01\n\x19GetRegisteredModelDetails\x12\x37\n\x10registered_model\x18\x01 \x01(\x0b\x32\x17.mlflow.RegisteredModelB\x04\xf8\x86\x19\x01\x1aN\n\x08Response\x12\x42\n\x19registered_model_detailed\x18\x01 \x01(\x0b\x32\x1f.mlflow.RegisteredModelDetailed:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x94\x01\n\x14ListRegisteredModels\x1aO\n\x08Response\x12\x43\n\x1aregistered_models_detailed\x18\x01 \x03(\x0b\x32\x1f.mlflow.RegisteredModelDetailed:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xd4\x01\n\x11GetLatestVersions\x12\x37\n\x10registered_model\x18\x01 \x01(\x0b\x32\x17.mlflow.RegisteredModelB\x04\xf8\x86\x19\x01\x12\x0e\n\x06stages\x18\x02 \x03(\t\x1aI\n\x08Response\x12=\n\x17model_versions_detailed\x18\x01 \x03(\x0b\x32\x1c.mlflow.ModelVersionDetailed:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb4\x01\n\x12\x43reateModelVersion\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x14\n\x06source\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x0e\n\x06run_id\x18\x03 \x01(\t\x1a\x37\n\x08Response\x12+\n\rmodel_version\x18\x01 \x01(\x0b\x32\x14.mlflow.ModelVersion:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xa4\x01\n\x12UpdateModelVersion\x12\x31\n\rmodel_version\x18\x01 \x01(\x0b\x32\x14.mlflow.ModelVersionB\x04\xf8\x86\x19\x01\x12\r\n\x05stage\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x80\x01\n\x12\x44\x65leteModelVersion\x12\x31\n\rmodel_version\x18\x01 \x01(\x0b\x32\x14.mlflow.ModelVersionB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xc2\x01\n\x16GetModelVersionDetails\x12\x31\n\rmodel_version\x18\x01 \x01(\x0b\x32\x14.mlflow.ModelVersionB\x04\xf8\x86\x19\x01\x1aH\n\x08Response\x12<\n\x16model_version_detailed\x18\x01 \x01(\x0b\x32\x1c.mlflow.ModelVersionDetailed:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xf6\x01\n\x13SearchModelVersions\x12\x0e\n\x06\x66ilter\x18\x01 \x01(\t\x12\x18\n\x0bmax_results\x18\x02 \x01(\x03:\x03\x31\x30\x30\x12\x10\n\x08order_by\x18\x03 \x03(\t\x12\x12\n\npage_token\x18\x04 \x01(\t\x1a\x62\n\x08Response\x12=\n\x17model_versions_detailed\x18\x01 \x03(\x0b\x32\x1c.mlflow.ModelVersionDetailed\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x93\x01\n\x15GetModelVersionStages\x12\x31\n\rmodel_version\x18\x01 \x01(\x0b\x32\x14.mlflow.ModelVersionB\x04\xf8\x86\x19\x01\x1a\x1a\n\x08Response\x12\x0e\n\x06stages\x18\x01 \x03(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x9e\x01\n\x1aGetModelVersionDownloadUri\x12\x31\n\rmodel_version\x18\x01 \x01(\x0b\x32\x14.mlflow.ModelVersionB\x04\xf8\x86\x19\x01\x1a \n\x08Response\x12\x14\n\x0c\x61rtifact_uri\x18\x01 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]*}\n\x12ModelVersionStatus\x12\x18\n\x14PENDING_REGISTRATION\x10\x01\x12\x17\n\x13\x46\x41ILED_REGISTRATION\x10\x02\x12\t\n\x05READY\x10\x03\x12\x14\n\x10PENDING_DELETION\x10\x04\x12\x13\n\x0f\x46\x41ILED_DELETION\x10\x05\x32\x8e\x13\n\x14ModelRegistryService\x12\xb6\x01\n\x15\x63reateRegisteredModel\x12\x1d.mlflow.CreateRegisteredModel\x1a&.mlflow.CreateRegisteredModel.Response\"V\xf2\x86\x19R\n6\n\x04POST\x12(/preview/mlflow/registered-models/create\x1a\x04\x08\x02\x10\x00\x10\x01*\x16\x43reate RegisteredModel\x12\xb7\x01\n\x15updateRegisteredModel\x12\x1d.mlflow.UpdateRegisteredModel\x1a&.mlflow.UpdateRegisteredModel.Response\"W\xf2\x86\x19S\n7\n\x05PATCH\x12(/preview/mlflow/registered-models/update\x1a\x04\x08\x02\x10\x00\x10\x01*\x16Update RegisteredModel\x12\xb8\x01\n\x15\x64\x65leteRegisteredModel\x12\x1d.mlflow.DeleteRegisteredModel\x1a&.mlflow.DeleteRegisteredModel.Response\"X\xf2\x86\x19T\n8\n\x06\x44\x45LETE\x12(/preview/mlflow/registered-models/delete\x1a\x04\x08\x02\x10\x00\x10\x01*\x16\x44\x65lete RegisteredModel\x12\xcc\x01\n\x19getRegisteredModelDetails\x12!.mlflow.GetRegisteredModelDetails\x1a*.mlflow.GetRegisteredModelDetails.Response\"`\xf2\x86\x19\\\n;\n\x04POST\x12-/preview/mlflow/registered-models/get-details\x1a\x04\x08\x02\x10\x00\x10\x01*\x1bGet RegisteredModel Details\x12\xaf\x01\n\x14listRegisteredModels\x12\x1c.mlflow.ListRegisteredModels\x1a%.mlflow.ListRegisteredModels.Response\"R\xf2\x86\x19N\n3\n\x03GET\x12&/preview/mlflow/registered-models/list\x1a\x04\x08\x02\x10\x00\x10\x01*\x15List RegisteredModels\x12\xb9\x01\n\x11getLatestVersions\x12\x19.mlflow.GetLatestVersions\x1a\".mlflow.GetLatestVersions.Response\"e\xf2\x86\x19\x61\nC\n\x04POST\x12\x35/preview/mlflow/registered-models/get-latest-versions\x1a\x04\x08\x02\x10\x00\x10\x01*\x18Get Latest ModelVersions\x12\xa7\x01\n\x12\x63reateModelVersion\x12\x1a.mlflow.CreateModelVersion\x1a#.mlflow.CreateModelVersion.Response\"P\xf2\x86\x19L\n3\n\x04POST\x12%/preview/mlflow/model-versions/create\x1a\x04\x08\x02\x10\x00\x10\x01*\x13\x43reate ModelVersion\x12\xa8\x01\n\x12updateModelVersion\x12\x1a.mlflow.UpdateModelVersion\x1a#.mlflow.UpdateModelVersion.Response\"Q\xf2\x86\x19M\n4\n\x05PATCH\x12%/preview/mlflow/model-versions/update\x1a\x04\x08\x02\x10\x00\x10\x01*\x13Update ModelVersion\x12\xa9\x01\n\x12\x64\x65leteModelVersion\x12\x1a.mlflow.DeleteModelVersion\x1a#.mlflow.DeleteModelVersion.Response\"R\xf2\x86\x19N\n5\n\x06\x44\x45LETE\x12%/preview/mlflow/model-versions/delete\x1a\x04\x08\x02\x10\x00\x10\x01*\x13\x44\x65lete ModelVersion\x12\xbd\x01\n\x16getModelVersionDetails\x12\x1e.mlflow.GetModelVersionDetails\x1a\'.mlflow.GetModelVersionDetails.Response\"Z\xf2\x86\x19V\n8\n\x04POST\x12*/preview/mlflow/model-versions/get-details\x1a\x04\x08\x02\x10\x00\x10\x01*\x18Get ModelVersion Details\x12\xaa\x01\n\x13searchModelVersions\x12\x1b.mlflow.SearchModelVersions\x1a$.mlflow.SearchModelVersions.Response\"P\xf2\x86\x19L\n2\n\x03GET\x12%/preview/mlflow/model-versions/search\x1a\x04\x08\x02\x10\x00\x10\x01*\x14Search ModelVersions\x12\xb8\x01\n\x15getModelVersionStages\x12\x1d.mlflow.GetModelVersionStages\x1a&.mlflow.GetModelVersionStages.Response\"X\xf2\x86\x19T\n7\n\x04POST\x12)/preview/mlflow/model-versions/get-stages\x1a\x04\x08\x02\x10\x00\x10\x03*\x17Get ModelVersion Stages\x12\xe1\x01\n\x1agetModelVersionDownloadUri\x12\".mlflow.GetModelVersionDownloadUri\x1a+.mlflow.GetModelVersionDownloadUri.Response\"r\xf2\x86\x19n\n=\n\x04POST\x12//preview/mlflow/model-versions/get-download-uri\x1a\x04\x08\x02\x10\x00\x10\x01*+Get Download URI For ModelVersion ArtifactsB!\n\x14org.mlflow.api.proto\x90\x01\x01\xa0\x01\x01\xe2?\x02\x10\x01')
  ,
  dependencies=[scalapb_dot_scalapb__pb2.DESCRIPTOR,databricks__pb2.DESCRIPTOR,])

_MODELVERSIONSTATUS = _descriptor.EnumDescriptor(
  name='ModelVersionStatus',
  full_name='mlflow.ModelVersionStatus',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PENDING_REGISTRATION', index=0, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAILED_REGISTRATION', index=1, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='READY', index=2, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PENDING_DELETION', index=3, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAILED_DELETION', index=4, number=5,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3049,
  serialized_end=3174,
)
_sym_db.RegisterEnumDescriptor(_MODELVERSIONSTATUS)

ModelVersionStatus = enum_type_wrapper.EnumTypeWrapper(_MODELVERSIONSTATUS)
PENDING_REGISTRATION = 1
FAILED_REGISTRATION = 2
READY = 3
PENDING_DELETION = 4
FAILED_DELETION = 5



_REGISTEREDMODEL = _descriptor.Descriptor(
  name='RegisteredModel',
  full_name='mlflow.RegisteredModel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='mlflow.RegisteredModel.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=73,
  serialized_end=104,
)


_REGISTEREDMODELDETAILED = _descriptor.Descriptor(
  name='RegisteredModelDetailed',
  full_name='mlflow.RegisteredModelDetailed',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered_model', full_name='mlflow.RegisteredModelDetailed.registered_model', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='creation_timestamp', full_name='mlflow.RegisteredModelDetailed.creation_timestamp', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='last_updated_timestamp', full_name='mlflow.RegisteredModelDetailed.last_updated_timestamp', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='user_id', full_name='mlflow.RegisteredModelDetailed.user_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='mlflow.RegisteredModelDetailed.description', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='latest_versions', full_name='mlflow.RegisteredModelDetailed.latest_versions', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=107,
  serialized_end=336,
)


_MODELVERSION = _descriptor.Descriptor(
  name='ModelVersion',
  full_name='mlflow.ModelVersion',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered_model', full_name='mlflow.ModelVersion.registered_model', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='mlflow.ModelVersion.version', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=338,
  serialized_end=420,
)


_MODELVERSIONDETAILED = _descriptor.Descriptor(
  name='ModelVersionDetailed',
  full_name='mlflow.ModelVersionDetailed',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_version', full_name='mlflow.ModelVersionDetailed.model_version', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='creation_timestamp', full_name='mlflow.ModelVersionDetailed.creation_timestamp', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='last_updated_timestamp', full_name='mlflow.ModelVersionDetailed.last_updated_timestamp', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='user_id', full_name='mlflow.ModelVersionDetailed.user_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='current_stage', full_name='mlflow.ModelVersionDetailed.current_stage', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='mlflow.ModelVersionDetailed.description', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='source', full_name='mlflow.ModelVersionDetailed.source', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='run_id', full_name='mlflow.ModelVersionDetailed.run_id', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='mlflow.ModelVersionDetailed.status', index=8,
      number=9, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status_message', full_name='mlflow.ModelVersionDetailed.status_message', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=423,
  serialized_end=711,
)


_CREATEREGISTEREDMODEL_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.CreateRegisteredModel.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered_model', full_name='mlflow.CreateRegisteredModel.Response.registered_model', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=759,
  serialized_end=820,
)

_CREATEREGISTEREDMODEL = _descriptor.Descriptor(
  name='CreateRegisteredModel',
  full_name='mlflow.CreateRegisteredModel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='mlflow.CreateRegisteredModel.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CREATEREGISTEREDMODEL_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=714,
  serialized_end=865,
)


_UPDATEREGISTEREDMODEL_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.UpdateRegisteredModel.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered_model', full_name='mlflow.UpdateRegisteredModel.Response.registered_model', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=759,
  serialized_end=820,
)

_UPDATEREGISTEREDMODEL = _descriptor.Descriptor(
  name='UpdateRegisteredModel',
  full_name='mlflow.UpdateRegisteredModel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered_model', full_name='mlflow.UpdateRegisteredModel.registered_model', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='mlflow.UpdateRegisteredModel.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='mlflow.UpdateRegisteredModel.description', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_UPDATEREGISTEREDMODEL_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=868,
  serialized_end=1091,
)


_DELETEREGISTEREDMODEL_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.DeleteRegisteredModel.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=759,
  serialized_end=769,
)

_DELETEREGISTEREDMODEL = _descriptor.Descriptor(
  name='DeleteRegisteredModel',
  full_name='mlflow.DeleteRegisteredModel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered_model', full_name='mlflow.DeleteRegisteredModel.registered_model', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_DELETEREGISTEREDMODEL_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1094,
  serialized_end=1231,
)


_GETREGISTEREDMODELDETAILS_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.GetRegisteredModelDetails.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered_model_detailed', full_name='mlflow.GetRegisteredModelDetails.Response.registered_model_detailed', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1320,
  serialized_end=1398,
)

_GETREGISTEREDMODELDETAILS = _descriptor.Descriptor(
  name='GetRegisteredModelDetails',
  full_name='mlflow.GetRegisteredModelDetails',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered_model', full_name='mlflow.GetRegisteredModelDetails.registered_model', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETREGISTEREDMODELDETAILS_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1234,
  serialized_end=1443,
)


_LISTREGISTEREDMODELS_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.ListRegisteredModels.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered_models_detailed', full_name='mlflow.ListRegisteredModels.Response.registered_models_detailed', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1470,
  serialized_end=1549,
)

_LISTREGISTEREDMODELS = _descriptor.Descriptor(
  name='ListRegisteredModels',
  full_name='mlflow.ListRegisteredModels',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[_LISTREGISTEREDMODELS_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1446,
  serialized_end=1594,
)


_GETLATESTVERSIONS_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.GetLatestVersions.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_versions_detailed', full_name='mlflow.GetLatestVersions.Response.model_versions_detailed', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1691,
  serialized_end=1764,
)

_GETLATESTVERSIONS = _descriptor.Descriptor(
  name='GetLatestVersions',
  full_name='mlflow.GetLatestVersions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='registered_model', full_name='mlflow.GetLatestVersions.registered_model', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stages', full_name='mlflow.GetLatestVersions.stages', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETLATESTVERSIONS_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1597,
  serialized_end=1809,
)


_CREATEMODELVERSION_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.CreateModelVersion.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_version', full_name='mlflow.CreateModelVersion.Response.model_version', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1892,
  serialized_end=1947,
)

_CREATEMODELVERSION = _descriptor.Descriptor(
  name='CreateModelVersion',
  full_name='mlflow.CreateModelVersion',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='mlflow.CreateModelVersion.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='source', full_name='mlflow.CreateModelVersion.source', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='run_id', full_name='mlflow.CreateModelVersion.run_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CREATEMODELVERSION_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1812,
  serialized_end=1992,
)


_UPDATEMODELVERSION_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.UpdateModelVersion.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=759,
  serialized_end=769,
)

_UPDATEMODELVERSION = _descriptor.Descriptor(
  name='UpdateModelVersion',
  full_name='mlflow.UpdateModelVersion',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_version', full_name='mlflow.UpdateModelVersion.model_version', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stage', full_name='mlflow.UpdateModelVersion.stage', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='mlflow.UpdateModelVersion.description', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_UPDATEMODELVERSION_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1995,
  serialized_end=2159,
)


_DELETEMODELVERSION_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.DeleteModelVersion.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=759,
  serialized_end=769,
)

_DELETEMODELVERSION = _descriptor.Descriptor(
  name='DeleteModelVersion',
  full_name='mlflow.DeleteModelVersion',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_version', full_name='mlflow.DeleteModelVersion.model_version', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_DELETEMODELVERSION_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2162,
  serialized_end=2290,
)


_GETMODELVERSIONDETAILS_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.GetModelVersionDetails.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_version_detailed', full_name='mlflow.GetModelVersionDetails.Response.model_version_detailed', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2370,
  serialized_end=2442,
)

_GETMODELVERSIONDETAILS = _descriptor.Descriptor(
  name='GetModelVersionDetails',
  full_name='mlflow.GetModelVersionDetails',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_version', full_name='mlflow.GetModelVersionDetails.model_version', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETMODELVERSIONDETAILS_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2293,
  serialized_end=2487,
)


_SEARCHMODELVERSIONS_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.SearchModelVersions.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_versions_detailed', full_name='mlflow.SearchModelVersions.Response.model_versions_detailed', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='next_page_token', full_name='mlflow.SearchModelVersions.Response.next_page_token', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2593,
  serialized_end=2691,
)

_SEARCHMODELVERSIONS = _descriptor.Descriptor(
  name='SearchModelVersions',
  full_name='mlflow.SearchModelVersions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='mlflow.SearchModelVersions.filter', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max_results', full_name='mlflow.SearchModelVersions.max_results', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=True, default_value=100,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='order_by', full_name='mlflow.SearchModelVersions.order_by', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='page_token', full_name='mlflow.SearchModelVersions.page_token', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_SEARCHMODELVERSIONS_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2490,
  serialized_end=2736,
)


_GETMODELVERSIONSTAGES_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.GetModelVersionStages.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stages', full_name='mlflow.GetModelVersionStages.Response.stages', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2815,
  serialized_end=2841,
)

_GETMODELVERSIONSTAGES = _descriptor.Descriptor(
  name='GetModelVersionStages',
  full_name='mlflow.GetModelVersionStages',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_version', full_name='mlflow.GetModelVersionStages.model_version', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETMODELVERSIONSTAGES_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2739,
  serialized_end=2886,
)


_GETMODELVERSIONDOWNLOADURI_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='mlflow.GetModelVersionDownloadUri.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='artifact_uri', full_name='mlflow.GetModelVersionDownloadUri.Response.artifact_uri', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2970,
  serialized_end=3002,
)

_GETMODELVERSIONDOWNLOADURI = _descriptor.Descriptor(
  name='GetModelVersionDownloadUri',
  full_name='mlflow.GetModelVersionDownloadUri',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_version', full_name='mlflow.GetModelVersionDownloadUri.model_version', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\370\206\031\001'), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETMODELVERSIONDOWNLOADURI_RESPONSE, ],
  enum_types=[
  ],
  serialized_options=_b('\342?(\n&com.databricks.rpc.RPC[$this.Response]'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2889,
  serialized_end=3047,
)

_REGISTEREDMODELDETAILED.fields_by_name['registered_model'].message_type = _REGISTEREDMODEL
_REGISTEREDMODELDETAILED.fields_by_name['latest_versions'].message_type = _MODELVERSIONDETAILED
_MODELVERSION.fields_by_name['registered_model'].message_type = _REGISTEREDMODEL
_MODELVERSIONDETAILED.fields_by_name['model_version'].message_type = _MODELVERSION
_MODELVERSIONDETAILED.fields_by_name['status'].enum_type = _MODELVERSIONSTATUS
_CREATEREGISTEREDMODEL_RESPONSE.fields_by_name['registered_model'].message_type = _REGISTEREDMODEL
_CREATEREGISTEREDMODEL_RESPONSE.containing_type = _CREATEREGISTEREDMODEL
_UPDATEREGISTEREDMODEL_RESPONSE.fields_by_name['registered_model'].message_type = _REGISTEREDMODEL
_UPDATEREGISTEREDMODEL_RESPONSE.containing_type = _UPDATEREGISTEREDMODEL
_UPDATEREGISTEREDMODEL.fields_by_name['registered_model'].message_type = _REGISTEREDMODEL
_DELETEREGISTEREDMODEL_RESPONSE.containing_type = _DELETEREGISTEREDMODEL
_DELETEREGISTEREDMODEL.fields_by_name['registered_model'].message_type = _REGISTEREDMODEL
_GETREGISTEREDMODELDETAILS_RESPONSE.fields_by_name['registered_model_detailed'].message_type = _REGISTEREDMODELDETAILED
_GETREGISTEREDMODELDETAILS_RESPONSE.containing_type = _GETREGISTEREDMODELDETAILS
_GETREGISTEREDMODELDETAILS.fields_by_name['registered_model'].message_type = _REGISTEREDMODEL
_LISTREGISTEREDMODELS_RESPONSE.fields_by_name['registered_models_detailed'].message_type = _REGISTEREDMODELDETAILED
_LISTREGISTEREDMODELS_RESPONSE.containing_type = _LISTREGISTEREDMODELS
_GETLATESTVERSIONS_RESPONSE.fields_by_name['model_versions_detailed'].message_type = _MODELVERSIONDETAILED
_GETLATESTVERSIONS_RESPONSE.containing_type = _GETLATESTVERSIONS
_GETLATESTVERSIONS.fields_by_name['registered_model'].message_type = _REGISTEREDMODEL
_CREATEMODELVERSION_RESPONSE.fields_by_name['model_version'].message_type = _MODELVERSION
_CREATEMODELVERSION_RESPONSE.containing_type = _CREATEMODELVERSION
_UPDATEMODELVERSION_RESPONSE.containing_type = _UPDATEMODELVERSION
_UPDATEMODELVERSION.fields_by_name['model_version'].message_type = _MODELVERSION
_DELETEMODELVERSION_RESPONSE.containing_type = _DELETEMODELVERSION
_DELETEMODELVERSION.fields_by_name['model_version'].message_type = _MODELVERSION
_GETMODELVERSIONDETAILS_RESPONSE.fields_by_name['model_version_detailed'].message_type = _MODELVERSIONDETAILED
_GETMODELVERSIONDETAILS_RESPONSE.containing_type = _GETMODELVERSIONDETAILS
_GETMODELVERSIONDETAILS.fields_by_name['model_version'].message_type = _MODELVERSION
_SEARCHMODELVERSIONS_RESPONSE.fields_by_name['model_versions_detailed'].message_type = _MODELVERSIONDETAILED
_SEARCHMODELVERSIONS_RESPONSE.containing_type = _SEARCHMODELVERSIONS
_GETMODELVERSIONSTAGES_RESPONSE.containing_type = _GETMODELVERSIONSTAGES
_GETMODELVERSIONSTAGES.fields_by_name['model_version'].message_type = _MODELVERSION
_GETMODELVERSIONDOWNLOADURI_RESPONSE.containing_type = _GETMODELVERSIONDOWNLOADURI
_GETMODELVERSIONDOWNLOADURI.fields_by_name['model_version'].message_type = _MODELVERSION
DESCRIPTOR.message_types_by_name['RegisteredModel'] = _REGISTEREDMODEL
DESCRIPTOR.message_types_by_name['RegisteredModelDetailed'] = _REGISTEREDMODELDETAILED
DESCRIPTOR.message_types_by_name['ModelVersion'] = _MODELVERSION
DESCRIPTOR.message_types_by_name['ModelVersionDetailed'] = _MODELVERSIONDETAILED
DESCRIPTOR.message_types_by_name['CreateRegisteredModel'] = _CREATEREGISTEREDMODEL
DESCRIPTOR.message_types_by_name['UpdateRegisteredModel'] = _UPDATEREGISTEREDMODEL
DESCRIPTOR.message_types_by_name['DeleteRegisteredModel'] = _DELETEREGISTEREDMODEL
DESCRIPTOR.message_types_by_name['GetRegisteredModelDetails'] = _GETREGISTEREDMODELDETAILS
DESCRIPTOR.message_types_by_name['ListRegisteredModels'] = _LISTREGISTEREDMODELS
DESCRIPTOR.message_types_by_name['GetLatestVersions'] = _GETLATESTVERSIONS
DESCRIPTOR.message_types_by_name['CreateModelVersion'] = _CREATEMODELVERSION
DESCRIPTOR.message_types_by_name['UpdateModelVersion'] = _UPDATEMODELVERSION
DESCRIPTOR.message_types_by_name['DeleteModelVersion'] = _DELETEMODELVERSION
DESCRIPTOR.message_types_by_name['GetModelVersionDetails'] = _GETMODELVERSIONDETAILS
DESCRIPTOR.message_types_by_name['SearchModelVersions'] = _SEARCHMODELVERSIONS
DESCRIPTOR.message_types_by_name['GetModelVersionStages'] = _GETMODELVERSIONSTAGES
DESCRIPTOR.message_types_by_name['GetModelVersionDownloadUri'] = _GETMODELVERSIONDOWNLOADURI
DESCRIPTOR.enum_types_by_name['ModelVersionStatus'] = _MODELVERSIONSTATUS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

RegisteredModel = _reflection.GeneratedProtocolMessageType('RegisteredModel', (_message.Message,), dict(
  DESCRIPTOR = _REGISTEREDMODEL,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.RegisteredModel)
  ))
_sym_db.RegisterMessage(RegisteredModel)

RegisteredModelDetailed = _reflection.GeneratedProtocolMessageType('RegisteredModelDetailed', (_message.Message,), dict(
  DESCRIPTOR = _REGISTEREDMODELDETAILED,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.RegisteredModelDetailed)
  ))
_sym_db.RegisterMessage(RegisteredModelDetailed)

ModelVersion = _reflection.GeneratedProtocolMessageType('ModelVersion', (_message.Message,), dict(
  DESCRIPTOR = _MODELVERSION,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.ModelVersion)
  ))
_sym_db.RegisterMessage(ModelVersion)

ModelVersionDetailed = _reflection.GeneratedProtocolMessageType('ModelVersionDetailed', (_message.Message,), dict(
  DESCRIPTOR = _MODELVERSIONDETAILED,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.ModelVersionDetailed)
  ))
_sym_db.RegisterMessage(ModelVersionDetailed)

CreateRegisteredModel = _reflection.GeneratedProtocolMessageType('CreateRegisteredModel', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _CREATEREGISTEREDMODEL_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.CreateRegisteredModel.Response)
    ))
  ,
  DESCRIPTOR = _CREATEREGISTEREDMODEL,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.CreateRegisteredModel)
  ))
_sym_db.RegisterMessage(CreateRegisteredModel)
_sym_db.RegisterMessage(CreateRegisteredModel.Response)

UpdateRegisteredModel = _reflection.GeneratedProtocolMessageType('UpdateRegisteredModel', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _UPDATEREGISTEREDMODEL_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.UpdateRegisteredModel.Response)
    ))
  ,
  DESCRIPTOR = _UPDATEREGISTEREDMODEL,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.UpdateRegisteredModel)
  ))
_sym_db.RegisterMessage(UpdateRegisteredModel)
_sym_db.RegisterMessage(UpdateRegisteredModel.Response)

DeleteRegisteredModel = _reflection.GeneratedProtocolMessageType('DeleteRegisteredModel', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _DELETEREGISTEREDMODEL_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteRegisteredModel.Response)
    ))
  ,
  DESCRIPTOR = _DELETEREGISTEREDMODEL,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.DeleteRegisteredModel)
  ))
_sym_db.RegisterMessage(DeleteRegisteredModel)
_sym_db.RegisterMessage(DeleteRegisteredModel.Response)

GetRegisteredModelDetails = _reflection.GeneratedProtocolMessageType('GetRegisteredModelDetails', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _GETREGISTEREDMODELDETAILS_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetRegisteredModelDetails.Response)
    ))
  ,
  DESCRIPTOR = _GETREGISTEREDMODELDETAILS,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.GetRegisteredModelDetails)
  ))
_sym_db.RegisterMessage(GetRegisteredModelDetails)
_sym_db.RegisterMessage(GetRegisteredModelDetails.Response)

ListRegisteredModels = _reflection.GeneratedProtocolMessageType('ListRegisteredModels', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _LISTREGISTEREDMODELS_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.ListRegisteredModels.Response)
    ))
  ,
  DESCRIPTOR = _LISTREGISTEREDMODELS,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.ListRegisteredModels)
  ))
_sym_db.RegisterMessage(ListRegisteredModels)
_sym_db.RegisterMessage(ListRegisteredModels.Response)

GetLatestVersions = _reflection.GeneratedProtocolMessageType('GetLatestVersions', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _GETLATESTVERSIONS_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetLatestVersions.Response)
    ))
  ,
  DESCRIPTOR = _GETLATESTVERSIONS,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.GetLatestVersions)
  ))
_sym_db.RegisterMessage(GetLatestVersions)
_sym_db.RegisterMessage(GetLatestVersions.Response)

CreateModelVersion = _reflection.GeneratedProtocolMessageType('CreateModelVersion', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _CREATEMODELVERSION_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.CreateModelVersion.Response)
    ))
  ,
  DESCRIPTOR = _CREATEMODELVERSION,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.CreateModelVersion)
  ))
_sym_db.RegisterMessage(CreateModelVersion)
_sym_db.RegisterMessage(CreateModelVersion.Response)

UpdateModelVersion = _reflection.GeneratedProtocolMessageType('UpdateModelVersion', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _UPDATEMODELVERSION_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.UpdateModelVersion.Response)
    ))
  ,
  DESCRIPTOR = _UPDATEMODELVERSION,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.UpdateModelVersion)
  ))
_sym_db.RegisterMessage(UpdateModelVersion)
_sym_db.RegisterMessage(UpdateModelVersion.Response)

DeleteModelVersion = _reflection.GeneratedProtocolMessageType('DeleteModelVersion', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _DELETEMODELVERSION_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteModelVersion.Response)
    ))
  ,
  DESCRIPTOR = _DELETEMODELVERSION,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.DeleteModelVersion)
  ))
_sym_db.RegisterMessage(DeleteModelVersion)
_sym_db.RegisterMessage(DeleteModelVersion.Response)

GetModelVersionDetails = _reflection.GeneratedProtocolMessageType('GetModelVersionDetails', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _GETMODELVERSIONDETAILS_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetModelVersionDetails.Response)
    ))
  ,
  DESCRIPTOR = _GETMODELVERSIONDETAILS,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.GetModelVersionDetails)
  ))
_sym_db.RegisterMessage(GetModelVersionDetails)
_sym_db.RegisterMessage(GetModelVersionDetails.Response)

SearchModelVersions = _reflection.GeneratedProtocolMessageType('SearchModelVersions', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _SEARCHMODELVERSIONS_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.SearchModelVersions.Response)
    ))
  ,
  DESCRIPTOR = _SEARCHMODELVERSIONS,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.SearchModelVersions)
  ))
_sym_db.RegisterMessage(SearchModelVersions)
_sym_db.RegisterMessage(SearchModelVersions.Response)

GetModelVersionStages = _reflection.GeneratedProtocolMessageType('GetModelVersionStages', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _GETMODELVERSIONSTAGES_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetModelVersionStages.Response)
    ))
  ,
  DESCRIPTOR = _GETMODELVERSIONSTAGES,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.GetModelVersionStages)
  ))
_sym_db.RegisterMessage(GetModelVersionStages)
_sym_db.RegisterMessage(GetModelVersionStages.Response)

GetModelVersionDownloadUri = _reflection.GeneratedProtocolMessageType('GetModelVersionDownloadUri', (_message.Message,), dict(

  Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
    DESCRIPTOR = _GETMODELVERSIONDOWNLOADURI_RESPONSE,
    __module__ = 'model_registry_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetModelVersionDownloadUri.Response)
    ))
  ,
  DESCRIPTOR = _GETMODELVERSIONDOWNLOADURI,
  __module__ = 'model_registry_pb2'
  # @@protoc_insertion_point(class_scope:mlflow.GetModelVersionDownloadUri)
  ))
_sym_db.RegisterMessage(GetModelVersionDownloadUri)
_sym_db.RegisterMessage(GetModelVersionDownloadUri.Response)


DESCRIPTOR._options = None
_CREATEREGISTEREDMODEL.fields_by_name['name']._options = None
_CREATEREGISTEREDMODEL._options = None
_UPDATEREGISTEREDMODEL.fields_by_name['registered_model']._options = None
_UPDATEREGISTEREDMODEL._options = None
_DELETEREGISTEREDMODEL.fields_by_name['registered_model']._options = None
_DELETEREGISTEREDMODEL._options = None
_GETREGISTEREDMODELDETAILS.fields_by_name['registered_model']._options = None
_GETREGISTEREDMODELDETAILS._options = None
_LISTREGISTEREDMODELS._options = None
_GETLATESTVERSIONS.fields_by_name['registered_model']._options = None
_GETLATESTVERSIONS._options = None
_CREATEMODELVERSION.fields_by_name['name']._options = None
_CREATEMODELVERSION.fields_by_name['source']._options = None
_CREATEMODELVERSION._options = None
_UPDATEMODELVERSION.fields_by_name['model_version']._options = None
_UPDATEMODELVERSION._options = None
_DELETEMODELVERSION.fields_by_name['model_version']._options = None
_DELETEMODELVERSION._options = None
_GETMODELVERSIONDETAILS.fields_by_name['model_version']._options = None
_GETMODELVERSIONDETAILS._options = None
_SEARCHMODELVERSIONS._options = None
_GETMODELVERSIONSTAGES.fields_by_name['model_version']._options = None
_GETMODELVERSIONSTAGES._options = None
_GETMODELVERSIONDOWNLOADURI.fields_by_name['model_version']._options = None
_GETMODELVERSIONDOWNLOADURI._options = None

_MODELREGISTRYSERVICE = _descriptor.ServiceDescriptor(
  name='ModelRegistryService',
  full_name='mlflow.ModelRegistryService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=3177,
  serialized_end=5623,
  methods=[
  _descriptor.MethodDescriptor(
    name='createRegisteredModel',
    full_name='mlflow.ModelRegistryService.createRegisteredModel',
    index=0,
    containing_service=None,
    input_type=_CREATEREGISTEREDMODEL,
    output_type=_CREATEREGISTEREDMODEL_RESPONSE,
    serialized_options=_b('\362\206\031R\n6\n\004POST\022(/preview/mlflow/registered-models/create\032\004\010\002\020\000\020\001*\026Create RegisteredModel'),
  ),
  _descriptor.MethodDescriptor(
    name='updateRegisteredModel',
    full_name='mlflow.ModelRegistryService.updateRegisteredModel',
    index=1,
    containing_service=None,
    input_type=_UPDATEREGISTEREDMODEL,
    output_type=_UPDATEREGISTEREDMODEL_RESPONSE,
    serialized_options=_b('\362\206\031S\n7\n\005PATCH\022(/preview/mlflow/registered-models/update\032\004\010\002\020\000\020\001*\026Update RegisteredModel'),
  ),
  _descriptor.MethodDescriptor(
    name='deleteRegisteredModel',
    full_name='mlflow.ModelRegistryService.deleteRegisteredModel',
    index=2,
    containing_service=None,
    input_type=_DELETEREGISTEREDMODEL,
    output_type=_DELETEREGISTEREDMODEL_RESPONSE,
    serialized_options=_b('\362\206\031T\n8\n\006DELETE\022(/preview/mlflow/registered-models/delete\032\004\010\002\020\000\020\001*\026Delete RegisteredModel'),
  ),
  _descriptor.MethodDescriptor(
    name='getRegisteredModelDetails',
    full_name='mlflow.ModelRegistryService.getRegisteredModelDetails',
    index=3,
    containing_service=None,
    input_type=_GETREGISTEREDMODELDETAILS,
    output_type=_GETREGISTEREDMODELDETAILS_RESPONSE,
    serialized_options=_b('\362\206\031\\\n;\n\004POST\022-/preview/mlflow/registered-models/get-details\032\004\010\002\020\000\020\001*\033Get RegisteredModel Details'),
  ),
  _descriptor.MethodDescriptor(
    name='listRegisteredModels',
    full_name='mlflow.ModelRegistryService.listRegisteredModels',
    index=4,
    containing_service=None,
    input_type=_LISTREGISTEREDMODELS,
    output_type=_LISTREGISTEREDMODELS_RESPONSE,
    serialized_options=_b('\362\206\031N\n3\n\003GET\022&/preview/mlflow/registered-models/list\032\004\010\002\020\000\020\001*\025List RegisteredModels'),
  ),
  _descriptor.MethodDescriptor(
    name='getLatestVersions',
    full_name='mlflow.ModelRegistryService.getLatestVersions',
    index=5,
    containing_service=None,
    input_type=_GETLATESTVERSIONS,
    output_type=_GETLATESTVERSIONS_RESPONSE,
    serialized_options=_b('\362\206\031a\nC\n\004POST\0225/preview/mlflow/registered-models/get-latest-versions\032\004\010\002\020\000\020\001*\030Get Latest ModelVersions'),
  ),
  _descriptor.MethodDescriptor(
    name='createModelVersion',
    full_name='mlflow.ModelRegistryService.createModelVersion',
    index=6,
    containing_service=None,
    input_type=_CREATEMODELVERSION,
    output_type=_CREATEMODELVERSION_RESPONSE,
    serialized_options=_b('\362\206\031L\n3\n\004POST\022%/preview/mlflow/model-versions/create\032\004\010\002\020\000\020\001*\023Create ModelVersion'),
  ),
  _descriptor.MethodDescriptor(
    name='updateModelVersion',
    full_name='mlflow.ModelRegistryService.updateModelVersion',
    index=7,
    containing_service=None,
    input_type=_UPDATEMODELVERSION,
    output_type=_UPDATEMODELVERSION_RESPONSE,
    serialized_options=_b('\362\206\031M\n4\n\005PATCH\022%/preview/mlflow/model-versions/update\032\004\010\002\020\000\020\001*\023Update ModelVersion'),
  ),
  _descriptor.MethodDescriptor(
    name='deleteModelVersion',
    full_name='mlflow.ModelRegistryService.deleteModelVersion',
    index=8,
    containing_service=None,
    input_type=_DELETEMODELVERSION,
    output_type=_DELETEMODELVERSION_RESPONSE,
    serialized_options=_b('\362\206\031N\n5\n\006DELETE\022%/preview/mlflow/model-versions/delete\032\004\010\002\020\000\020\001*\023Delete ModelVersion'),
  ),
  _descriptor.MethodDescriptor(
    name='getModelVersionDetails',
    full_name='mlflow.ModelRegistryService.getModelVersionDetails',
    index=9,
    containing_service=None,
    input_type=_GETMODELVERSIONDETAILS,
    output_type=_GETMODELVERSIONDETAILS_RESPONSE,
    serialized_options=_b('\362\206\031V\n8\n\004POST\022*/preview/mlflow/model-versions/get-details\032\004\010\002\020\000\020\001*\030Get ModelVersion Details'),
  ),
  _descriptor.MethodDescriptor(
    name='searchModelVersions',
    full_name='mlflow.ModelRegistryService.searchModelVersions',
    index=10,
    containing_service=None,
    input_type=_SEARCHMODELVERSIONS,
    output_type=_SEARCHMODELVERSIONS_RESPONSE,
    serialized_options=_b('\362\206\031L\n2\n\003GET\022%/preview/mlflow/model-versions/search\032\004\010\002\020\000\020\001*\024Search ModelVersions'),
  ),
  _descriptor.MethodDescriptor(
    name='getModelVersionStages',
    full_name='mlflow.ModelRegistryService.getModelVersionStages',
    index=11,
    containing_service=None,
    input_type=_GETMODELVERSIONSTAGES,
    output_type=_GETMODELVERSIONSTAGES_RESPONSE,
    serialized_options=_b('\362\206\031T\n7\n\004POST\022)/preview/mlflow/model-versions/get-stages\032\004\010\002\020\000\020\003*\027Get ModelVersion Stages'),
  ),
  _descriptor.MethodDescriptor(
    name='getModelVersionDownloadUri',
    full_name='mlflow.ModelRegistryService.getModelVersionDownloadUri',
    index=12,
    containing_service=None,
    input_type=_GETMODELVERSIONDOWNLOADURI,
    output_type=_GETMODELVERSIONDOWNLOADURI_RESPONSE,
    serialized_options=_b('\362\206\031n\n=\n\004POST\022//preview/mlflow/model-versions/get-download-uri\032\004\010\002\020\000\020\001*+Get Download URI For ModelVersion Artifacts'),
  ),
])
_sym_db.RegisterServiceDescriptor(_MODELREGISTRYSERVICE)

DESCRIPTOR.services_by_name['ModelRegistryService'] = _MODELREGISTRYSERVICE

ModelRegistryService = service_reflection.GeneratedServiceType('ModelRegistryService', (_service.Service,), dict(
  DESCRIPTOR = _MODELREGISTRYSERVICE,
  __module__ = 'model_registry_pb2'
  ))

ModelRegistryService_Stub = service_reflection.GeneratedServiceStubType('ModelRegistryService_Stub', (ModelRegistryService,), dict(
  DESCRIPTOR = _MODELREGISTRYSERVICE,
  __module__ = 'model_registry_pb2'
  ))


# @@protoc_insertion_point(module_scope)
